from __future__ import annotations

import typing

from solders.instruction import AccountMeta, Instruction
from solders.pubkey import Pubkey
from solders.system_program import ID as SYS_PROGRAM_ID

from ..program_id import PROGRAM_ID


class CreateEscrowAccountAccounts(typing.TypedDict):
    escrow: Pubkey
    metadata: Pubkey
    mint: Pubkey
    token_account: Pubkey
    edition: Pubkey
    payer: Pubkey
    sysvar_instructions: Pubkey
    authority: typing.Optional[Pubkey]


def create_escrow_account(
    accounts: CreateEscrowAccountAccounts,
    program_id: Pubkey = PROGRAM_ID,
    remaining_accounts: typing.Optional[typing.List[AccountMeta]] = None,
) -> Instruction:
    keys: list[AccountMeta] = [
        AccountMeta(pubkey=accounts["escrow"], is_signer=False, is_writable=True),
        AccountMeta(pubkey=accounts["metadata"], is_signer=False, is_writable=True),
        AccountMeta(pubkey=accounts["mint"], is_signer=False, is_writable=False),
        AccountMeta(
            pubkey=accounts["token_account"], is_signer=False, is_writable=False
        ),
        AccountMeta(pubkey=accounts["edition"], is_signer=False, is_writable=False),
        AccountMeta(pubkey=accounts["payer"], is_signer=True, is_writable=True),
        AccountMeta(pubkey=SYS_PROGRAM_ID, is_signer=False, is_writable=False),
        AccountMeta(
            pubkey=accounts["sysvar_instructions"], is_signer=False, is_writable=False
        ),
        (
            AccountMeta(pubkey=accounts["authority"], is_signer=True, is_writable=False)
            if accounts["authority"]
            else AccountMeta(pubkey=program_id, is_signer=False, is_writable=False)
        ),
    ]
    if remaining_accounts is not None:
        keys += remaining_accounts
    identifier = b"\x92\x93\xe1/3@p\x01"
    encoded_args = b""
    data = identifier + encoded_args
    return Instruction(program_id, data, keys)
